/**************************************************************** 
 * Description: 2010NOIP  T3  关押罪犯 并查集方法
 * Author: Alex Li
 * Date: 2024-08-13 09:38:49
 * LastEditTime: 2024-08-13 09:55:37
****************************************************************/

#include <iostream>
#include <algorithm>
using namespace std;

// 定义一个结构体用于存储罪犯之间的仇恨信息
struct data {
    int x, y, z; //z代码x罪犯和y罪犯之间的怨气气值
};

// 声明全局变量
data f[100005];  // 存储所有仇恨对的信息
int n, m, a[20005], b[20005], i;

// 比较函数，用于对仇恨对按照怨气值从大到小排序
inline bool cmp(data a, data b) {
    return a.z > b.z;
}

// 查找函数，用于找到某个罪犯的“根”并进行路径压缩
inline int find(int x) {
    if (a[x] == x) return x;  // 如果自己是根，直接返回
    a[x] = find(a[x]);  // 路径压缩
    return a[x];
}

// 合并函数，将两个罪犯归为同一组
inline void ad(int x, int y) {
    x = find(a[x]);
    y = find(a[y]);
    a[x] = y;  // 合并两个罪犯的根
}

// 检查函数，判断两个罪犯是否在同一监狱（同一组）
inline bool check(int x, int y) {
    x = find(x);
    y = find(y);
    return x == y;  // 如果他们的根相同，则在同一组
}

int main() {
    // 输入罪犯数量和仇恨对数量
    cin >> n >> m;

    // 初始化，每个罪犯的根都是他自己
    for (i = 1; i <= n; i++) a[i] = i;

    // 输入所有的仇恨对信息
    for (i = 1; i <= m; i++)
        cin >> f[i].x >> f[i].y >> f[i].z;

    // 按照仇恨值从大到小排序
    sort(f + 1, f + m + 1, cmp);

    // 依次处理每一对仇恨对
    for (i = 1; i <= m + 1; i++) {
        // 如果这对罪犯已经在同一监狱（同一组），输出他们的仇恨值，并退出
        if (check(f[i].x, f[i].y)) {
            cout << f[i].z;
            break;
        } else {
            // 否则，合并这对仇恨对的敌人关系
            if (!b[f[i].x]) b[f[i].x] = f[i].y;  // 标记敌人
            else ad(b[f[i].x], f[i].y);  // 将敌人的敌人合并
            if (!b[f[i].y]) b[f[i].y] = f[i].x;
            else ad(b[f[i].y], f[i].x);
        }
    }

    return 0;
}

/*================================================================
代码解释：
结构体 data： 用于存储罪犯之间的仇恨对，包含罪犯编号 x、y 和他们之间的怨气值 z。

find 函数： 使用路径压缩技术查找并返回罪犯的根节点（代表其所在的组）。如果一个罪犯的根节点就是它自己，返回自己，否则递归查找它的根并进行路径压缩。

ad 函数： 将两个罪犯的根节点合并，从而将这两个罪犯归为同一组。

check 函数： 判断两个罪犯是否在同一组。如果两个罪犯的根节点相同，则表示他们在同一监狱（同一组）。

cmp 函数： 用于对仇恨对按照怨气值从大到小排序，这样我们可以从最大怨气值开始处理。

主要逻辑：

输入罪犯的数量和仇恨对的数量，并初始化每个罪犯的根节点。
输入每一对罪犯之间的仇恨信息，并按照怨气值从大到小排序。
遍历排序后的仇恨对：
如果这对罪犯已经在同一监狱（同一组），输出他们的怨气值并结束程序。
否则，将他们的敌人关系进行合并，确保敌人的敌人归为同一组。
如果没有冲突发生，程序会输出 0。

在这段代码中，并查集的主要用途是：

查找（Find）：使用 find 函数找到某个罪犯的“根”节点，判断该罪犯属于哪个集合。根节点代表一个集合中的所有元素。

合并（Union）：使用 ad 函数将两个罪犯所在的集合合并为一个集合。

判断连接性：使用 check 函数来判断两个罪犯是否属于同一个集合，如果两个罪犯在同一个集合中，意味着他们已经被关在同一个监狱里。

通过对所有的仇恨对按照怨气值从大到小排序，然后依次将这些仇恨对进行处理，代码最终输出的就是在所有的仇恨对中，Z市长看到的第一个冲突事件的影响力最小的情况。

所以，这个问题的解决方案确实是基于并查集（Union-Find）来进行的。





*/